"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrestodbQuery = void 0;
const shared_1 = require("@cubejs-backend/shared");
const BaseQuery_1 = require("./BaseQuery");
const BaseFilter_1 = require("./BaseFilter");
const GRANULARITY_TO_INTERVAL = {
    day: 'day',
    week: 'week',
    hour: 'hour',
    minute: 'minute',
    second: 'second',
    month: 'month',
    quarter: 'quarter',
    year: 'year'
};
class PrestodbFilter extends BaseFilter_1.BaseFilter {
    likeIgnoreCase(column, not, param, type) {
        const p = (!type || type === 'contains' || type === 'ends') ? '%' : '';
        const s = (!type || type === 'contains' || type === 'starts') ? '%' : '';
        return `LOWER(${column})${not ? ' NOT' : ''} LIKE CONCAT('${p}', LOWER(${this.allocateParam(param)}) , '${s}') ESCAPE '\\'`;
    }
    castParameter() {
        if (this.definition().type === 'boolean') {
            return 'CAST(? AS BOOLEAN)';
        }
        else if (this.measure || this.definition().type === 'number') {
            // TODO here can be measure type of string actually
            return 'CAST(? AS DOUBLE)';
        }
        return '?';
    }
}
class PrestodbQuery extends BaseQuery_1.BaseQuery {
    newFilter(filter) {
        return new PrestodbFilter(this, filter);
    }
    timeStampParam() {
        return 'from_iso8601_timestamp(?)';
    }
    timeStampCast(value) {
        return `from_iso8601_timestamp(${value})`;
    }
    dateTimeCast(value) {
        return `from_iso8601_timestamp(${value})`;
    }
    convertTz(field) {
        const atTimezone = `${field} AT TIME ZONE '${this.timezone}'`;
        return this.timezone ?
            `CAST(date_add('minute', timezone_minute(${atTimezone}), date_add('hour', timezone_hour(${atTimezone}), ${field})) AS TIMESTAMP)` :
            field;
    }
    /**
     * Returns sql for source expression floored to timestamps aligned with
     * intervals relative to origin timestamp point.
     * Athena doesn't support INTERVALs directly — using date_diff/date_add
     */
    dateBin(interval, source, origin) {
        const intervalParsed = (0, shared_1.parseSqlInterval)(interval);
        const intervalParts = Object.entries(intervalParsed);
        if (intervalParts.length > 1) {
            throw new Error('Athena/Presto supports only simple intervals with one date part');
        }
        const [unit, count] = intervalParts[0];
        const originExpr = this.timeStampCast(`'${origin}'`);
        return `date_add('${unit}',
      floor(
        date_diff('${unit}', ${originExpr}, ${source}) / ${count}
      ) * ${count},
      ${originExpr}
    )`;
    }
    timeGroupedColumn(granularity, dimension) {
        return `date_trunc('${GRANULARITY_TO_INTERVAL[granularity]}', ${dimension})`;
    }
    intervalString(interval) {
        const [intervalValue, intervalUnit] = interval.split(' ');
        return `'${intervalValue}' ${intervalUnit}`;
    }
    seriesSql(timeDimension) {
        const values = timeDimension.timeSeries().map(([from, to]) => `select '${from}' f, '${to}' t`).join(' UNION ALL ');
        return `SELECT from_iso8601_timestamp(dates.f) date_from, from_iso8601_timestamp(dates.t) date_to FROM (${values}) AS dates`;
    }
    unixTimestampSql() {
        return `to_unixtime(${this.nowTimestampSql()})`;
    }
    defaultRefreshKeyRenewalThreshold() {
        return 120;
    }
    defaultEveryRefreshKey() {
        return {
            every: '2 minutes'
        };
    }
    hllInit(sql) {
        return `cast(approx_set(${sql}) as varbinary)`;
    }
    hllMerge(sql) {
        return `cardinality(merge(cast(${sql} as HyperLogLog)))`;
    }
    countDistinctApprox(sql) {
        return `approx_distinct(${sql})`;
    }
    limitOffsetClause(limit, offset) {
        const limitClause = limit != null ? ` LIMIT ${limit}` : '';
        const offsetClause = offset != null ? ` OFFSET ${offset}` : '';
        return `${offsetClause}${limitClause}`;
    }
    sqlTemplates() {
        const templates = super.sqlTemplates();
        templates.functions.DATETRUNC = 'DATE_TRUNC({{ args_concat }})';
        templates.functions.DATEPART = 'DATE_PART({{ args_concat }})';
        delete templates.functions.PERCENTILECONT;
        templates.statements.select = '{% if ctes %} WITH \n' +
            '{{ ctes | join(\',\n\') }}\n' +
            '{% endif %}' +
            'SELECT {{ select_concat | map(attribute=\'aliased\') | join(\', \') }}  {% if from %}\n' +
            'FROM (\n  {{ from }}\n) AS {{ from_alias }} {% elif from_prepared %}\n' +
            'FROM {{ from_prepared }}' +
            '{% endif %}' +
            '{% if filter %}\nWHERE {{ filter }}{% endif %}' +
            '{% if group_by %} GROUP BY {{ group_by }}{% endif %}' +
            '{% if having %}\nHAVING {{ having }}{% endif %}' +
            '{% if order_by %} ORDER BY {{ order_by | map(attribute=\'expr\') | join(\', \') }}{% endif %}' +
            '{% if offset is not none %}\nOFFSET {{ offset }}{% endif %}' +
            '{% if limit is not none %}\nLIMIT {{ limit }}{% endif %}';
        templates.expressions.extract = 'EXTRACT({{ date_part }} FROM {{ expr }})';
        templates.expressions.interval_single_date_part = 'INTERVAL \'{{ num }}\' {{ date_part }}';
        templates.expressions.timestamp_literal = 'from_iso8601_timestamp(\'{{ value }}\')';
        delete templates.expressions.ilike;
        templates.types.string = 'VARCHAR';
        templates.types.float = 'REAL';
        // Presto intervals have a YearMonth or DayTime type variants, but no universal type
        delete templates.types.interval;
        templates.types.binary = 'VARBINARY';
        templates.tesseract.ilike = 'LOWER({{ expr }}) {% if negated %}NOT {% endif %} LIKE {{ pattern }}';
        templates.filters.like_pattern = 'CONCAT({% if start_wild %}\'%\'{% else %}\'\'{% endif %}, LOWER({{ value }}), {% if end_wild %}\'%\'{% else %}\'\'{% endif %}) ESCAPE \'\\\'';
        templates.statements.time_series_select = 'SELECT from_iso8601_timestamp(dates.f) date_from, from_iso8601_timestamp(dates.t) date_to \n' +
            'FROM (\n' +
            '{% for time_item in seria  %}' +
            '    select \'{{ time_item[0] }}\' f, \'{{ time_item[1] }}\' t \n' +
            '{% if not loop.last %} UNION ALL\n{% endif %}' +
            '{% endfor %}' +
            ') AS dates';
        templates.statements.generated_time_series_select = 'SELECT d AS date_from,\n' +
            'date_add(\'MILLISECOND\', -1, d + interval {{ granularity }}) AS date_to\n' +
            'FROM UNNEST(\n' +
            'SEQUENCE(CAST(from_iso8601_timestamp({{ start }}) AS TIMESTAMP), CAST(from_iso8601_timestamp({{ end }}) AS TIMESTAMP), INTERVAL {{ granularity }})\n' +
            ') AS dates(d)';
        templates.statements.generated_time_series_with_cte_range_source = 'SELECT d AS date_from,\n' +
            'date_add(\'MILLISECOND\', -1, d + interval {{ granularity }}) AS date_to\n' +
            'FROM {{ range_source }} CROSS JOIN UNNEST(\n' +
            'SEQUENCE(CAST({{ range_source }}.{{ min_name }} AS TIMESTAMP), CAST({{ range_source }}.{{ max_name }} AS TIMESTAMP), INTERVAL {{ granularity }})\n' +
            ') AS dates(d)';
        return templates;
    }
    castToString(sql) {
        return `CAST(${sql} as VARCHAR)`;
    }
}
exports.PrestodbQuery = PrestodbQuery;
//# sourceMappingURL=PrestodbQuery.js.map